function LabelAxes(hax,root,subscripts,varargin)
% function LabelAxes(hax,root,subscripts,varargin)
%    label axes from cell array of labels. Works from 1 to 3 dimensions
% labels are root with subscripts
%    inputs:
%        hax: (handle) current axis handle
%        subscripts: (cell array of strings) the label subscripts
%        root: (string) the label root
%        fontsize: (optional = 12), the label fontsize
%    outputs:
%        none
%    example:
%      % labels are A_{soft tissue},A_{bone},A_{iodine}
%    subscripts = {'soft tissue','bone','iodine'};
%    LabelAxes(gca,subscripts,'root','A');
%    REA 12/Mar/2015 10:23 - 20/Mar/2015 11:04
%    COPYRIGHT:  Robert E. Alvarez, 2015. All rights reserved.
  
nreqargs = 2;
assert(nargin>=nreqargs);
if ~ishandle(hax)
    error('hax parameter must be a valid handle');
end
if ~ischar(root)
    error('root parameter must be string')
end
if iscell(subscripts)
    assert(iscellstr(subscripts));
    assert(length(subscripts)<=3);
elseif isvector(subscripts) && ischar(subscripts)
    subscripts = {subscripts}; % convert string to cell array
else
    error('subscripts parameter must be cell array or string');
end

fontsiz = 12;

if(nargin>nreqargs)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'fontsize';          fontsiz =varargin{i+1}; i=i+1;
    otherwise
      error('Unknown argument %s given',varargin{i});
     end
     i=i+1;
  end
end

  % could not find way to refer to axis labels using an index
  % hence clumsy switch construct
axes(hax); % the labels will apply to this axis
for kaxis = 1:length(subscripts)
    label = sprintf('%s_{%s}',root,subscripts{kaxis});
    switch kaxis
        case 1; xlabel(label,'fontsize',fontsiz);
        case 2; ylabel(label,'fontsize',fontsiz);
        case 3; zlabel(label,'fontsize',fontsiz);
        otherwise
           error('Length of subscripts %d. only 1:3 axes can be labeled', ...
               length(subscripts));
    end
end
