% P62Est3D_3.m Estimator for contrast-3
%
% COPYRIGHT:  Aprend Technology and Robert E. ALvarez, 2016. All rights reserved.
 
%% init vars
post_title = 'P62Est3D_3';
% modify this for your computer
blog_dir_name = 'L:\Projects\Blog\Posts\';
addpath([blog_dir_name post_title filesep 'code']);
addpath([blog_dir_name 'Template']);
addpath('L:\Projects\Downloads\altmany-export_fig-eaeef72');
% MakeBlogPostFileLists(post_title);

%% init path variables--modify for your computer
dirname2code = 'L:\Projects\DEsolvePaper\';
    % set path to paper code to access Atable estimator function
p = genpath(dirname2code);
addpath(p);
dirname_figures = [blog_dir_name post_title filesep];

%% Create the spectrum
tube_voltage = 120; % 100; % % kV  
[~,~,specdat] = XrayTubeSpectrumTasmip(tube_voltage,'number_spectrum','clipzeros');

%% mass attenuation coefficient data
nbasis = 3;
materials = {'C5O2H8','Al','C31H64','I'}; % acrylic plastic, aluminum, iodine contrast agent base
mus = MusFromMaterials(materials,specdat.egys); 
   % the attenuation coefficient of the contrast agent simulant 
   % combines the plastic matrix 'C31H64' plus the iodine attenuation coefficients
iodine_frac = 0.2;
mus(:,nbasis) = (1-iodine_frac)*mus(:,nbasis) + iodine_frac*mus(:,nbasis+1); % The simulant attenuation coefficient
specdat.mus = mus(:,1:nbasis); % throw away last column after using it in the previous line
A_names = cell(1,nbasis); % names of A-vector components
materials_names = {'plastic','Al','iodine'}; 
for kmatl = 1:nbasis
    A_names{kmatl} = sprintf('A_{%s}',materials_names{kmatl});
end

%% make the  points on the three lines
thicknesses_max = [20 1.5 0.125]; % gm/cm^2 of 'soft_tissue','bone_compact' iodine 
   % compute end points of lines
fracs =  [0.8*ones(1,3); ...
    .25 .6 .9;  ...
    .8 .25 .8 ];
endpoints_lines = bsxfun(@times,fracs,thicknesses_max); % the endpoints
nlines = size(endpoints_lines,1);       
npoints_on_line = 25;% number of points along line
   % make equal spaced points on lines thru A-space
Alines = linspaceNDim(zeros(size(endpoints_lines)),endpoints_lines,npoints_on_line);   

%% Set threshold energies for PHA bins
nbins = 5; %3; % % NOTE: nbins  specified here <-----
specfilt = specdat;
  % find optimum bins at specified fraction of maximum thicknesses
specfilt.specnum = specfilt.specnum.*exp(-0.2*specfilt.mus*thicknesses_max');
    % try out different delta A vectors
    % shows result does not depend on them
dA = [0 0 -1]'; % column vector
for kbasis = 1:numel(dA) 
    dA = circshift(dA,1); 
    specdat.idx_threshold = OptimNbinsThresholdsSNR3(specfilt,nbins,'dA',dA);
    disp([dA' specdat.egys(specdat.idx_threshold)'])
end

%% make the calibration phantom data
calib_thicknesses_max = thicknesses_max; % step wedge thicknesses to span the object
ncalib_steps = 30;
t1s = geospace(0,calib_thicknesses_max(1),ncalib_steps);  
t2s =  geospace(0,calib_thicknesses_max(2),ncalib_steps); 
t3s = geospace(0,calib_thicknesses_max(3),ncalib_steps); 
[X,Y,Z] = meshgrid(t1s,t2s,t3s);
cdat.As = [X(:),Y(:),Z(:)];
assert(all(cdat.As(1,:)==0)); % so can use these for zero thickness (air) values 
cdat.lambdas = PoissonLambda(cdat.As, specdat); % lambdas are Poisson expected values--see paper
   % compute data for calibration by averaging random data
   % as would be done in implementation
use_random_calib = true;
if use_random_calib
    ncalib_trials = 300; % number of exposures to average together to get "low noise" data
       % compute index to step in center of calibration phantom for noise covariance calculation
       % choose step that has closest Avector to center of phantom
    A_in_center = 0.4*calib_thicknesses_max;    
       % implement the following using bsxfun
%     [~,idx2step4cov] = min(normrow(cdat.As - A_in_center));
    difference_As4steps = bsxfun(@minus,cdat.As, A_in_center(:)');
    [~,idx2step4cov] = min(normrow(difference_As4steps)); % closest step to desired A value
      % compute data covariance as cov of average of ncalib_trials
    fprintf('start compute calibration data. ntrials = %d\n',ncalib_trials);
    ns4ave_accum = zeros(size(cdat.lambdas)); % accumulate data for average number of photons calc. 
    ns4cov = zeros(ncalib_trials,nbins); % accumulate data for covariance calc
    lambdas_calibrator = cdat.lambdas;
    lambdas_calibrator_devs = sqrt(lambdas_calibrator); % std dev of counts
    assert(min(lambdas_calibrator(:))>100); % so can use normal approx to Poisson, much faster than poissrnd
    for ktrial = 1:ncalib_trials
        ns4trial = lambdas_calibrator + lambdas_calibrator_devs.*randn(size(lambdas_calibrator));
%         ns4trial = poissrnd(lambdas_calibrator);
        ns4ave_accum = ns4ave_accum + ns4trial;
        ns4cov(ktrial,:) = ns4trial(idx2step4cov,:);
    end
    fprintf('finished calibrator data calculation\n');
    cdat.ns = ns4ave_accum/ncalib_trials; % average of accumulated data for trials
    cdat.logI = -bsxfun(@minus, log(cdat.ns), log(lambdas_calibrator(1,:)));
    logI4cov = -bsxfun(@minus, log(ns4cov), log(lambdas_calibrator(1,:)));
    cdat.RL = cov(logI4cov);% covariance used to compute linear MLE from random data
else
    cdat.ns = cdat.lambdas;
    cdat.logI = -bsxfun(@minus,log(cdat.ns), cdat.lambdas(1,:));
        % covariance used to compute linear MLE. compute in interior of calibration region
    cdat.RL = diag(1./PoissonLambda(0.6*calib_thicknesses_max, specdat));   
end

%% get random data and estimates for points on 3 lines through A-space
   % this code cell can be used for all types of estimators including iterative
estimators2use = {'Atable','Iterative'};   % estimators to compare
estimatorsMap = containers.Map(estimators2use,1:length(estimators2use)); % use to refer to data by name
nestimators = length(estimators2use);
    % set up the spectrum
nphotons = 1e6;
specdat.specnum = nphotons*specdat.specnum_norm; 
   % Monte Carlo parameters
rng('default'); % set random number gen to default state so get same results as in paper figures   
nrand_trials = 2000; % % number of random trials per point
idx_test = repmat((1:npoints_on_line)',nrand_trials,1); % duplicate points on line for extract data in random trials

   % initialize the estimator parameter structs--will be replaced by actual
   % the first time the estimator functions called.
solvedatAtable = [];
solvedatIterative = [];
sdats = struct([]); % struct array to hold results for each estimator and line
fprintf('starting Monte Carlo simulation. This will take a long time\n');
for kline = 1:nlines
    fprintf('start line %d\n',kline);
       % get the Avectors of the line
    As_line_actual = squeeze(Alines(kline,:,:))';
        % generate random data
    lambdas = PoissonLambda(As_line_actual,specdat); % expected values
    nphotons_expected_values = repmat(lambdas,nrand_trials,1); 
    ns_random = poissrnd(nphotons_expected_values); 
    random_data_zeros = (ns_random(:)==0); % array of logicals
    if any(random_data_zeros)
        warning('some random counts are zero');
        fprintf('kline %d frac of zeros %g\n',kline,sum(random_data_zeros)/numel(ns_random));
        ns_random(random_data_zeros) = nphotons_expected_values(random_data_zeros); % force no zeros so can take log
    end
       % prep the test data struct
    tdat_noise.As = repmat(As_line_actual,nrand_trials,1); % the As for each line
       
       % compute noisy logI data from random photon counts ns_random
       % in following, use fact that first point of line is for zero thickness
    tdat_noise.logI = -bsxfun(@minus, log(ns_random), log(lambdas(1,:)));
    tdat_noise.ns = ns_random; 
    tdat_no_noise.As = tdat_noise.As; % copy data in struct 
    tdat_no_noise.logI = -bsxfun(@minus, log(nphotons_expected_values), log(lambdas(1,:))); %then change to no-noise logI 
    for kestimator = 1:nestimators
        estimator2use = estimators2use{kestimator};
        switch lower(estimator2use)
            case lower('Atable')
                fprintf('start Atable estimator\n');
                tic
                if isempty(solvedatAtable) % compute estimator data struct first time only
                    dat_noise = AtableSolveEquations3(cdat,tdat_noise);
                    solvedatAtable = dat_noise.solvedat;
                    dat_no_noise = AtableSolveEquations3(cdat,tdat_no_noise,'solvedat',solvedatAtable);
                else % for later calls, use pre-computed estimator data struct
                    dat_noise =  AtableSolveEquations3(cdat,tdat_noise,'solvedat',solvedatAtable);
                    dat_no_noise = AtableSolveEquations3(cdat,tdat_no_noise,'solvedat',solvedatAtable);
                end
                fprintf('line %d Atable time %g\n',kline,toc);
            case lower('Iterative')
                fprintf('start iterative estimator\n');
                if isempty(solvedatIterative)
                    sdat = AsolveIterFromSpectrum(specdat,tdat_noise,'init_vals',tdat_noise.As);
                    solvedatIterative = sdat.solvedat;
                else
                    sdat = AsolveIterFromSpectrum(specdat,tdat_noise,'init_vals',tdat_noise.As,'solvedat',solvedatIterative);
                end
                dat_noise = sdat;
                dat_no_noise = []; % do not process no-noise data--too much time already
            otherwise
                error('Estimator %s not found',estimator2use);
        end
        s.dat_noise = dat_noise;
        s.dat_no_noise = dat_no_noise;
            % add result to summary struct array
        sdats = AddStruct2StructArray(sdats,s);
    end
end
  % reshape sdats array for easier access
sdats = reshape(sdats,nestimators,nlines); 
   %
if any(strcmpi(estimators2use,'iterative')) % print the total times if using iterative estimator
    fprintf('times_ratio %g\n',sum(sdats(2,1).dat_noise.times)/sdats(1,1).dat_noise.time);
end
for k=1:5, beep; pause(0.1); end

%% display the A-space lines with no-noise and noisy estimates for a selected estimator
estimator2display = 'Iterative'; %'Atable'; % 
kest = estimatorsMap(estimator2display);
colors = 'kbr';
plot_symbols = '+*d';
fh = figure;
set(fh,'color','w','position',[150 150 350 1050])
titles = {'(a) no-noise','(b) line 1','(c) line 2','(d) line 3'};
nrandom_points2display = 1:npoints_on_line; % number of random points to display
  
subplot(4,1,1) % first no-noise reconstruction
hs_actual = [];
hs_no_noise = [];
hold on   
for kl = 1:nlines
    As = squeeze(Alines(kl,:,:))';
    hs_actual(kl) = plot3(As(:,1),As(:,2),As(:,3),['-' colors(kl)],'markersize',2,'linewidth',1.5);
    if ~isempty(sdats(kest,kl).dat_no_noise) % handle iterative case where no-noise not done
        Astest = sdats(kest,kl).dat_no_noise.Astest(1:npoints_on_line,:);
        hs_no_noise(kl) = plot3(Astest(:,1),Astest(:,2),Astest(:,3),['+' colors(kl)],'markersize',4,'linewidth',1.5);
    end
        % label the line 1:3
    plabel = 1.05*(As(end,:)-As(1,:)); % extrapolated point off end of line for label
    ht = text(plabel(1),plabel(2),plabel(3),sprintf('%d',kl),'fontsize',12);
end
hold off 
axis([0   20.0000         0    2.0000         0    0.1500]);
set(gca,'fontsize',12,'linewidth',1.5,'cameraposition',[-28 -15 .4]);
grid on
LabelAxes(gca,'A', materials_names);
title(titles{1},'fontsize',12);

hs_actual = [];
for kline = 1:3
    subplot(4,1,kline+1)
    title(titles{kline+1},'fontsize',12);
    Astest = sdats(kest,kline).dat_noise.Astest(nrandom_points2display,:);
    hold on
       % plot random points
    hnoise = plot3(Astest(:,1),Astest(:,2),Astest(:,3),['+' colors(kline)],'markersize',8);
      % add actual lines
    hs_actual = zeros(1,nlines);
    for kl = 1:nlines
        As = squeeze(Alines(kl,:,:))';
        hs_actual(kl) = plot3(As(:,1),As(:,2),As(:,3),['-' colors(kl)],'markersize',2,'linewidth',1.5);
    end
    hold off
    set(gca,'fontsize',12,'linewidth',1.5,'cameraposition',[-28 -15 .4]);
      % no negative on axis scales
    axis([0   20.0000         0    2.0000         0    0.1500]);    grid on
    LabelAxes(gca,'A',materials_names);
end
hann = annotation(fh,'textbox',...
    [0.4 0.95 0.215 0.032],'String',estimator2display, ...
    'FitBoxToText','off','fontsize',13,'edgecolor','none');
    
%% export the figure as png and eps
fname =sprintf('AplaneLines%s%dbins',estimator2display,nbins);
%    % export as eps
figure(fh)   
% export_fig([dirname_figures fname '.eps'],'-eps');

%% overlap plot of the estimates for two estimators
estimators2display = {'Atable','Iterative'};
estimator_number = zeros(length(estimators2display),1);
for k = 1:length(estimators2display)
    estimator_number(k) = estimatorsMap(estimators2display{k});
end
colors = 'kbr';
plot_symbols = '+o';
titles = {'line 1','line 2','line 3'};
nrandom_points2display = 1:npoints_on_line; % number of random points to display

hs_actual = [];
lines2plot = 1;
fh = figure;
if numel(lines2plot==1) % set figure size and position for different number subplots
    psn = [150   150   525   470];
else
    psn = [150 150 350 1050];
end
set(fh,'color','w','position',psn)
Astest4estimators = zeros(numel(estimator_number),numel(nrandom_points2display),nbasis);
for kplot = 1:numel(lines2plot)
    subplot(numel(lines2plot),1,kplot)
    kline = lines2plot(kplot);
    title(titles{kline},'fontsize',12);
    hold on
    for kestimator = 1:numel(estimator_number)
        kest = estimator_number(kestimator);
        Astest = sdats(kest,kline).dat_noise.Astest(nrandom_points2display,:);
        Astest4estimators(kestimator,:,:) = Astest;
           % plot random points
        hnoise = plot3(Astest(:,1),Astest(:,2),Astest(:,3),[plot_symbols(kestimator) colors(kline)],'markersize',8);
    end
    hl = legend(estimators2display);
      % add actual lines
    hs_actual = zeros(1,nlines);
    for kl = 1:nlines
        As = squeeze(Alines(kl,:,:))';
        hs_actual(kl) = plot3(As(:,1),As(:,2),As(:,3),['-' colors(kl)],'markersize',2,'linewidth',1.5);
    end
    hold off
    set(gca,'fontsize',12,'linewidth',1.5,'cameraposition',[-28 -15 .4]);
      % no negative on axis scales
    axis([0   20.0000         0    2.0000         0    0.1500]);    grid on
    LabelAxes(gca,'A',materials_names);
end

%% export the figure as png and eps
fname =sprintf('EstimatesOverlapPlot%dbins',nbins);
   % export as eps
figure(fh)   
% export_fig([dirname_figures fname '.eps'],'-eps');

%% plot normalized Atable - Iterative estimates
   % use diff in the following since Astest4estimators has values for both
   % Atable and Iterative. Parms for diff are order of diff and dimension
normalized_diff = normrow(squeeze(diff(Astest4estimators,1,1)))/norm(squeeze(Astest4estimators(1,end,:)));
% normalized_diff = normrow(squeeze(diff(Astest4estimators,1,1)))./normrow(squeeze(Astest4estimators(1,:,:)));
fh = figure;
plot(1:npoints_on_line,normalized_diff,'+k','markersize',10);
xlabel('point number','fontsize',13);
ylabel('norm(Atable-Iterative)/norm(A(end))','fontsize',13);
set(gca,'fontsize',12,'linewidth',1.5,'yscale','log')
set(fh,'color','w','position',psn)
labelPointsZ(gca,zcomp([(1:npoints_on_line)',normalized_diff]),'fontsize',8)


%% export the figure as png and eps
fname =sprintf('EstimatesOverlapPlot2D%dbins',nbins);
   % export as eps
figure(fh)   
% export_fig([dirname_figures fname '.eps'],'-eps');

%% extract the estimates' variance, MSE
  % variables to hold results
CR_var_all = zeros(nlines,npoints_on_line,nbasis,nestimators);
means_all = zeros(nlines,npoints_on_line,nbasis,nestimators);
var_all = zeros(nlines,npoints_on_line,nbasis,nestimators); % Monte Carlo variance
mse_all = zeros(nlines,npoints_on_line,nbasis,nestimators); % Monte Carlo mean squared error
cov_CRLB_diff_norm = zeros(nlines,npoints_on_line,nestimators);
for kestimator = 1:nestimators
    for kline = 1:nlines
        dat_noise = sdats(kestimator,kline).dat_noise;
              % process the random estimates for each point in the line
             % prep for CRLB calc
        As_line_actual = squeeze(Alines(kline,:,:))';
            % get expected values for PHA bin counts
        lambdas = PoissonLambda(As_line_actual,specdat); % expected values
        nphotons_expected_values = repmat(lambdas,nrand_trials,1); 
        Ms_all = mubarNbinNdim(specdat,As_line_actual);
        for kt = 1:npoints_on_line
                    % first the mean, variance and mean squared errors of the data
               idx = find(kt==idx_test); % index to random estimates for this point on line
               As4point = dat_noise.Astest(idx,:); % random A vectors for this point on line
               means_all(kline,kt,:,kestimator) = mean(As4point); % means at each point
               var_all(kline,kt,:,kestimator) = var(As4point); % variance
               mse_all(kline,kt,:,kestimator) = mean(dat_noise.Asdiff(idx,:).^2); % mean squared error
                  % then the CR bound -- use constant covariance approximation   
              M = Ms_all(:,:,kt);
              lambdas_pnt = nphotons_expected_values(kt,:);
              RLinv = diag(lambdas_pnt);
              RA_CR = inv(M'*RLinv*M);
              CR_var_all(kline,kt,:,kestimator) = diag(RA_CR)'; 
              cov_CRLB_diff_norm(kline,kt,kestimator) = norm(cov(As4point) - RA_CR)/norm(RA_CR);
        end
    end
end

%% make a scatterplot of variances and CRLB or std devs vs distance from center
fh = figure;
set(fh,'color','white','position',[100 100 700 900])
plot_symbols = {'d','*','o'};
colors = 'kbr';
hax = zeros(nbasis);
h_plots = [];
error_type = 'MSE'; %'variance'; % 
p = 1; % subplot position
estimators2plot = 1:length(estimators2display); % 1; %[1 2];
for kline = 1:nlines
    thicknesses = sqrt(sum((squeeze(Alines(kline,:,:))').^2,2));
    for kbasis = 1:nbasis
        color = colors(kline);
        subplot(nlines,nbasis,p)
        p = p + 1;
        CR_var = squeeze(CR_var_all(kline,:,:,estimators2plot(1))); 
        h_plots{end+1} = semilogy(thicknesses,CR_var(:,kbasis),['-' color],'linewidth',1.5);
        errs = []; %prep to set range
        hold on 
        for kestimator = estimators2plot
            switch lower(error_type)
                case 'mse';
                 mse_dat = squeeze(mse_all(kline,:,:,kestimator));            
                 h_plots{end+1} = semilogy(thicknesses(1:2:end),mse_dat(1:2:end,kbasis), ...
                     [plot_symbols{kestimator} color]);
                 errs = [errs; mse_dat(:,kbasis)];
                case 'variance';
                 var_dat = squeeze(var_all(kline,:,:,kestimator));
                 h_plots{end+1} = semilogy(thicknesses,var_dat(:,kbasis), ...
                     [plot_symbols{kestimator} color]);
                 errs = [errs; var_dat(:,kbasis)];
                case 'std'
                    std_dat = squeeze(mu_normalized_stds(kline,:,kestimator));
                    if kbasis==1
                        h_plots{end+1} = plot(thicknesses,std_dat, ...
                            [plot_symbols{kestimator} color]);
                    end
                otherwise
                    error('unrecognized error type');
            end
        end
        hold off
        [mn,mx] = range2([errs(:);CR_var(:,kbasis)]);
        ax = [0 max(thicknesses) 10^floor(log10(mn)) 10^ceil(log10(mx))];
        axis(ax);
        hax(kbasis) = gca;
        set(gca,'fontsize',13','linewidth',1.5)
        if kbasis==2 && kline==3
            xlabel('A vector magnitude (gm/cm^2)','fontsize',13);
        end
        if kbasis==1 && kline==2
            ylabel(sprintf('%s (gm/cm^2)^2',error_type),'fontsize',13);
        end
        if kline==1
            title(A_names{kbasis},'fontsize',13);
            if kbasis == 2
                text(0.1*ax(2),0.4*ax(4),sprintf('%d bins PHA',nbins),'fontsize',13);
            end
        end
        if kline==2 && kbasis==3
            legends_all = {'CRLB' estimators2display{estimators2plot}};
            hl = legend(legends_all,'location','south','fontsize',10);
        end
        if kbasis == 1
             text(0.1*max(thicknesses),10^ceil(log10(mx)-1),sprintf('line %d',kline),'fontsize',13);
        end
    end
end

%% export the figure as eps
if strcmp(error_type,'MSE')
    fname = sprintf('MSEvsCRLB%dbins',nbins);
else
    fname = sprintf('VARvsCRLB%dbins',nbins);
end
   % export as eps
% if any(strcmpi(estimators2display,'iterative'))
%     fname = [fname 'iterative'];
% end
figure(fh)
export_fig([dirname_figures fname '.eps'],'-eps');


%% draw optimal energy bins
   % create high egy-resolution data so do not get slanted line in mu-plot
egyshr = fliplr(linspace(min(specdat.egys),max(specdat.egys),1000)); % high res energies
% mu_ref = MusFromMaterials('soft_tissue',egyshr); 
Imu_hres = xraymu('I',egyshr);
fh = figure;
set(fh,'color','white','position',[ 100 100 580 375])
hold on
   % first draw the spectrum
plot(specdat.egys, 100*specdat.specnum_norm/max(specdat.specnum_norm),'-.k','linewidth',1.5);
   % superimpose the iodine mu  
plot(egyshr,300*Imu_hres/max(Imu_hres),'-r','linewidth',1.5); % use 300 to clip off the low energy values
hold off
axis([0 max(specdat.egys) 0.1 100]);
xlabel('energy (keV)','fontsize',12);
Esall = [1; specdat.egys(specdat.idx_threshold); max(specdat.egys)];
ys = [100; 100];
harr = zeros(5,1);
for kbin = 1:nbins
    xs = [Esall(kbin); Esall(kbin+1)];
    zs = zcomp([xs ys]);
    hl = line([xs(1) xs(1)],[0.1 100],'LineStyle','--','linewidth',1);
    if kbin==5
        text(0.98*mean(xs),10,num2str(kbin),'fontsize',12);
    else
        text(mean(xs),10,num2str(kbin),'fontsize',12)
    end
    harr(kbin) = zarrow2head(zs(1),zs(2),'color','r');
    psn = get(harr(kbin),'position');
    set(harr(kbin),'position',[psn(1) 0.3 psn(3:end)]);
end
set(gca,'fontsize',12','linewidth',1.5);

%% export the figure as png and eps
fname = 'PHAbins';
   % export as eps
% export_fig(fh,[dirname_figures fname '.eps'],'-eps');

%% display convex hull of As_MLE
dat_table = sdats(1,1).dat_noise;
As_MLE = dat_table.solvedat.As_MLE_calibrator;
X = As_MLE;
K = convhulln(X);
fh = figure;
set(fh,'position',[200 200 570 1300],'color','w')

subplot(4,1,1)
htri = trisurf(K,X(:,1),X(:,2),X(:,3));
set(htri,'facealpha',0.7,'EdgeColor',0.8*[1 1 1]);
colormap gray
xlabel('A_{MLE,1} (g/cm^2)','fontsize',12)
ylabel('A_{MLE,2} (g/cm^2)','fontsize',12)
zlabel('A_{MLE,3} (g/cm^2)','fontsize',12)
set(gca,'linewidth',1,'fontsize',12);

   % draw projections on axes
idxs = (1:3)';
for kprj = 1:3
    subplot(4,1,kprj+1)   
    x = X( K(:,idxs(1)),idxs(1));
    y = X( K(:,idxs(2)),idxs(2)); % select out columns
    Kxy = convhull(x,y);
    fill(x(Kxy),y(Kxy),0.8*[1 1 1])
%     plot(x(Kxy),y(Kxy),'-k','linewidth',1.5)
    xlabel(sprintf('A_{MLE,%d}',idxs(1)),'fontsize',14);
    ylabel(sprintf('A_{MLE,%d}',idxs(2)),'fontsize',14);
    set(gca,'linewidth',1.2,'fontsize',14)
    idxs = circshift(idxs,1);
end

%% export fig
fname = '3Dconvhull.eps';
figure(fh)
% export_fig([dirname_figures fname],'-eps');
