function varargout = labelPointsZ(axisHandle, zs2label,varargin)
% function h_labels = labelPointsZ(axisHandle, zs2label,varargin)
%  Label points in a plot with their indices
% Arguments:
%  axisHandle    Typically use gca
%  zs2label: location of points to label. Will use index of this array as label
%    unless labels optional argument used below
%  optional arguments:
%    color: 'k'     Text color - string such as 'r' or rgb val such as [1 0 0]
%    fontsize: 12     fontsize for labels
%    xshift_frac: 0.01 fraction of axis width to offset the labels from the points 
%    labels: (indexes of zs2label) either numerical array or cell array of strings
% Returned:
%  h_labels      A vector of handles to the plotted text labels
%
% 11/18/07 REA 

min_arg_req = 2;
assert(nargin>=min_arg_req);
color = 'k';
labels = [];
fontsize = 12; 
xshift_frac = 0.01;
yshift_frac = 0;

if(nargin>min_arg_req)
  i=1;
  while(i<=size(varargin,2))
     switch lower(varargin{i})
     case 'color';        color =varargin{i+1};     i=i+1;  
     case 'fontsize';        fontsize =varargin{i+1};     i=i+1;  
     case 'labels';        labels =varargin{i+1};     i=i+1;  
     case 'xshift_frac';        xshift_frac =varargin{i+1};     i=i+1;  
     case 'yshift_frac';        yshift_frac =varargin{i+1};     i=i+1;  
     otherwise
        error('Unknown argument %s',varargin{i});
     end
     i=i+1;
  end
end


if isempty(labels)
  labels = cellstr(num2str([1:numel(zs2label)]'));
end

if isnumeric(labels)
    zs2label = zs2label(labels);
    labels = cellstr(num2str(labels(:)));
end

h_labels = text(real(zs2label),imag(zs2label),labels,'VerticalAlignment','bottom','HorizontalAlignment','right', ...
        'Color',color,'fontsize',fontsize,'Parent',axisHandle);


if nargout>=1
    varargout{1} = h_labels;
end
